#include "DxLib.h"
#include "image.hpp"
#include "tilemap.hpp"
#include "stagecontext.hpp"
#include <cstdint>
#include <memory>

/*
 * }bvf[^
 */

static uint8_t mapdata_layer1[8*8] = {
	0,0,0,1,1,0,0,0,
	0,0,1,1,1,1,0,0,
	0,1,1,1,1,1,1,0,
	1,1,0,1,1,0,1,1,
	1,1,1,1,1,1,1,1,
	0,0,1,0,0,1,0,0,
	0,1,0,1,1,0,1,0,
	1,0,1,0,0,1,0,1,
};
static uint8_t mapdata_layer2[8*8] = {
	0,0,1,0,0,0,0,0,
	0,0,0,1,0,0,0,1,
	0,0,1,1,1,1,1,1,
	0,1,1,0,1,1,1,0,
	1,1,1,1,1,1,1,1,
	1,0,1,1,1,1,1,1,
	1,0,1,0,0,0,0,0,
	0,0,0,1,1,0,1,1,
};
static uint8_t mapdata_layer3[8*8] = {
	0,0,0,0,1,1,1,1,
	0,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,
	1,1,1,0,0,1,1,0,
	1,1,1,1,1,1,1,1,
	0,0,0,1,1,0,0,1,
	0,0,1,1,0,1,1,0,
	1,1,0,0,0,0,0,0,
};

// 1}X̃^C
struct TileInfo {
	int layer1;
	int layer2;
	int layer3;
};

/*
 * `p̃IuWFNg
 */

class DungeonTileImageList : public TileImageList<TileInfo>
{
public:
	std::vector<std::shared_ptr<Image>> vec;
	int GetTileWidth() { return 32; }
	int GetTileHeight() { return 32; }
	std::shared_ptr<Image> GetImage(TileInfo &info) {
		class Tile : public Image {
		private:
			int width, height;
			DungeonTileImageList *parent;
			TileInfo &info;
		public:
			Tile(int width, int height, DungeonTileImageList *parent, TileInfo &info)
			: width(width), height(height), parent(parent), info(info) {}
			void Draw(int x, int y) {
				if (info.layer3) parent->vec[3]->Draw(x, y);
				if (info.layer2) parent->vec[2]->Draw(x+1, y+1);
				if (info.layer1) parent->vec[1]->Draw(x+2, y+2);
			}
		};
		return std::make_shared<Tile>(32, 32, this, info);
	}
};
static std::shared_ptr<TileImageList<TileInfo>> imglst;

class DungeonTileIndexer : public TileIndexer<TileInfo>
{
	uint8_t *mapdata_layer1;
	uint8_t *mapdata_layer2;
	uint8_t *mapdata_layer3;
public:
	DungeonTileIndexer(uint8_t *mapdata_layer1, uint8_t *mapdata_layer2, uint8_t *mapdata_layer3)
	: mapdata_layer1(mapdata_layer1)
	, mapdata_layer2(mapdata_layer2)
	, mapdata_layer3(mapdata_layer3) {}
	TileInfo GetTileInfo(int x, int y) {
		TileInfo info;
		info.layer1 = mapdata_layer1[y*GetMapWidth()+x];
		info.layer2 = mapdata_layer2[y*GetMapWidth()+x];
		info.layer3 = mapdata_layer3[y*GetMapWidth()+x];
		return info;
	}
	int GetMapWidth() { return 8; }
	int GetMapHeight() { return 8; }
};
static std::shared_ptr<TileIndexer<TileInfo>> tileidxr;

static std::shared_ptr<Tilemap<TileInfo>> tilemap;

/*
 * Cxgp̃IuWFNg
 */

class DungeonMapEventListener : public MapEventListener
{
public:
	void ActionEvent(int x, int y, MapEvent e) {
		switch (e.kind) {
		case MapEvent::Wall_Break: /* ǂ */
			if (x >= 0 && x < 8 && y >= 0 && y < 8) {
				/* ォ珇ɕǂΏ */
				if (mapdata_layer1[y*8+x]) {
					mapdata_layer1[y*8+x] = 0;
				} else if (mapdata_layer2[y*8+x]) {
					mapdata_layer2[y*8+x] = 0;
				} else if (mapdata_layer3[y*8+x]) {
					mapdata_layer3[y*8+x] = 0;
				}
			}
		}
	}
};
static std::shared_ptr<MapEventListener> listener;

void dungeonMake()
{
	/*
	 * }bv𐶐
	 */
	// ()

	/*
	 * `p^C}bv
	 */
	/* C[WXg */
	auto imglst1 = std::make_shared<DungeonTileImageList>();
	// Ƃ肠vectorɒړ˂
	imglst1->vec.push_back(MakeColorImage(32, 32, GetColor(0,0,0)));
	imglst1->vec.push_back(MakeColorImage(28, 28, GetColor(128,255,128)));
	imglst1->vec.push_back(MakeColorImage(30, 30, GetColor(128,128,255)));
	imglst1->vec.push_back(MakeColorImage(32, 32, GetColor(255,128,128)));
	imglst = imglst1;
	/* CfNT(ɖOƍĂȂ) */
	tileidxr = std::make_shared<DungeonTileIndexer>(mapdata_layer1, mapdata_layer2, mapdata_layer3);
	/* ^C}bv */
	tilemap = std::make_shared<Tilemap<TileInfo>>(256, 256, *tileidxr, *imglst);

	/*
	 * Cxg̃R[obNIuWFNgo^
	 */
	listener = std::make_shared<DungeonMapEventListener>();
	GetStageContext().setMapEventListener(listener);
}

void dungeonDraw()
{
	tilemap->Draw(0, 0);
	/*
	  ǗIuWFNgɗD揇ʕtŕ`IuWFNgo^
	  ǗIuWFNgꊇĕ`悷dg݂ɂ
	  ʂ̕`揈͕svɂȂ

	  }bv̉ŃLB悤ȗ̓Iȕ\̏ꍇłA
	  `IuWFNg𕪂ēo^Ηǂ
	  }bvf[^̂̂ɑ΂鏈ɂ͉eȂ
	*/
}
