#include <vector>
#include <memory>

template<class T>
class TileImageList
{
public:
	virtual int GetTileWidth() = 0;
	virtual int GetTileHeight() = 0;
	virtual std::shared_ptr<Image> GetImage(T &info) = 0;
};

template<class T>
class TileIndexer
{
public:
	virtual T GetTileInfo(int x, int y) = 0;
	virtual int GetMapWidth() = 0;
	virtual int GetMapHeight() = 0;
};

template<class T>
class Tilemap : public Image
{
private:
	TileIndexer<T> &idxr;
	TileImageList<T> &imglst;
public:
	Tilemap(int w, int h, TileIndexer<T> &idxr, TileImageList<T> &imglst)
	: idxr(idxr), imglst(imglst) {}
	void Draw(int x, int y) {
		int mw = idxr.GetMapWidth();
		int mh = idxr.GetMapHeight();
		int tx = x, ty = y;
		for (int i=0; i<mh; i++) {
			for (int j=0; j<mw; j++) {
				imglst.GetImage(idxr.GetTileInfo(j, i))->Draw(tx, ty);
				tx += imglst.GetTileWidth();
			}
			ty += imglst.GetTileHeight();
			tx = x;
		}
	}
};

